﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.IO;
using System.Web;
using Antlr3.ST;
using Commons.Collections;
using NVelocity;
using NVelocity.App;
using NVelocity.Runtime;

public class TemplateUtil
{
    private static string ST_ROOT_PATH = string.Empty;
    static TemplateUtil()
    {
        string rootDir = HttpContext.Current.Server.MapPath("./st");
        if (rootDir.EndsWith("\\"))
            rootDir = rootDir.Substring(0, rootDir.Length - 1);
        ST_ROOT_PATH = rootDir;
    }
    public static void WriteTime(string title, long time)
    {
        HttpContext.Current.Response.Write("<br />" + title);
        long millisecondes = time % 1000;
        long minutes = time / 1000 / 60;
        long seconds = (time - minutes * 60 * 1000) / 1000;
        if (minutes > 0)
            HttpContext.Current.Response.Write(minutes.ToString() + "m, ");
        if (seconds > 0)
            HttpContext.Current.Response.Write(seconds.ToString() + "s, ");
        HttpContext.Current.Response.Write(millisecondes.ToString() + "ms");
    }
    public static long NVelocityTest(int loops)
    {
        VelocityEngine engine = new VelocityEngine();
        ExtendedProperties prop = new ExtendedProperties();
        prop.AddProperty(RuntimeConstants.FILE_RESOURCE_LOADER_PATH
            , HttpContext.Current.Server.MapPath("./"));
        prop.AddProperty(RuntimeConstants.ENCODING_DEFAULT, "utf-8");
        engine.Init(prop);

        Stopwatch sw = new Stopwatch();
        sw.Start();
        for (int i = 0; i < loops; i++)
            NVelocityParse(engine, i, false);
        sw.Stop();
        WriteTime("Total Elapsed Time: ", sw.ElapsedMilliseconds);
        return sw.ElapsedMilliseconds;
    }
    public static void NVelocityParse(VelocityEngine engine, int loop, bool output)
    {
        Template template = engine.GetTemplate("contact-list.vm");
        VelocityContext context = new VelocityContext();
        context.Put("contacts", new List<Contact>()
        {
            new Contact() { 
                Name="Richie Liu", TotalOrderAmt=180,
                EMail="RichieLiu-test@gmail.com", 
                Address="上海徐汇区钦州路" },
            new Contact() { 
                Name="Kevin Zhang",TotalOrderAmt=0, 
                EMail="KevinZhang-test@gmail.com", 
                Address="苏州新区" },
            new Contact() { 
                Name="Eric Wong", TotalOrderAmt=626,
                EMail="EricWong-test@gmail.com", 
                Address="厦门海沧" },
            new Contact() { 
                Name="RicCC", TotalOrderAmt=2080,
                EMail="RicCC-test@gmail.com", 
                Address="上海徐汇区钦州路" }
        });
        context.Put("qryName", "Ric");
        context.Put("qryAddress", DateTime.Now);
        context.Put("hasMessage", true);
        context.Put("message", "This is a message from the server.");

        context.Put("items1", new string[] { "AAA", "BBB", "CCC" });
        context.Put("items2"
            , new List<DateTime> { new DateTime(1979, 1, 1), new DateTime(2010, 4, 1) });

        IDictionary<string, decimal> dic = new Dictionary<string, decimal>();
        dic.Add("AAA", 111M);
        dic.Add("BBB", 222M);
        dic.Add("CCC", 333M);
        context.Put("items3", dic);

        DataTable table = new DataTable();
        table.Columns.Add(new DataColumn("Key", typeof(string)));
        table.Columns.Add(new DataColumn("Value", typeof(int)));
        DataRow row = table.NewRow();
        row["Key"] = "item 1";
        row["Value"] = 111;
        table.Rows.Add(row);
        row = table.NewRow();
        row["Key"] = "item 2";
        row["Value"] = 222;
        table.Rows.Add(row);
        context.Put("items4", table);

        if (!output)
        {
            StringWriter writer = new StringWriter();
            template.Merge(context, writer);
            writer.Close();
        }
        else
            template.Merge(context, HttpContext.Current.Response.Output);
    }
    public static long StringTemplateTest(int loops)
    {
        Stopwatch sw = new Stopwatch();
        sw.Start();
        for (int i = 0; i < loops; i++)
            StringTemplateParse(i, false);
        sw.Stop();
        WriteTime("Total Elapsed Time: ", sw.ElapsedMilliseconds);
        return sw.ElapsedMilliseconds;
    }
    public static void StringTemplateParse(int loop, bool output)
    {
        StringTemplateGroup group = new StringTemplateGroup("stperf", ST_ROOT_PATH);

        StringTemplate st = group.LookupTemplate("contact_list");
        st.SetAttribute("contacts", new List<Contact>()
        {
            new Contact() { 
                Name="Richie Liu", TotalOrderAmt=180,
                EMail="RichieLiu-test@gmail.com", 
                Address="上海徐汇区钦州路" },
            new Contact() { 
                Name="Kevin Zhang",TotalOrderAmt=0, 
                EMail="KevinZhang-test@gmail.com", 
                Address="苏州新区" },
            new Contact() { 
                Name="Eric Wong", TotalOrderAmt=626,
                EMail="EricWong-test@gmail.com", 
                Address="厦门海沧" },
            new Contact() { 
                Name="RicCC", TotalOrderAmt=2080,
                EMail="RicCC-test@gmail.com", 
                Address="上海徐汇区钦州路" }
        });
        st.SetAttribute("qryName", "Ric");
        st.SetAttribute("qryAddress", DateTime.Now);
        st.SetAttribute("hasMessage", true);
        st.SetAttribute("message", "This is a message from the server.");

        st.SetAttribute("items1", new string[] { "AAA", "BBB", "CCC" });
        st.SetAttribute("items2"
            , new List<DateTime> { new DateTime(1979, 1, 1), new DateTime(2010, 4, 1) });

        Dictionary<string, decimal> dic = new Dictionary<string, decimal>();
        dic.Add("AAA", 111M);
        dic.Add("BBB", 222M);
        dic.Add("CCC", 333M);
        st.SetAttribute("items3", dic);

        DataTable table = new DataTable();
        table.Columns.Add(new DataColumn("Key1", typeof(string)));
        table.Columns.Add(new DataColumn("Value1", typeof(int)));
        DataRow row = table.NewRow();
        row["Key1"] = "item 1";
        row["Value1"] = 111;
        table.Rows.Add(row);
        row = table.NewRow();
        row["Key1"] = "item 2";
        row["Value1"] = 222;
        table.Rows.Add(row);
        st.SetAttribute("items4", table);

        if (!output) st.ToString();
        else HttpContext.Current.Response.Write(st.ToString());
    }
    public static long ASPXTest(int loops)
    {
        Stopwatch sw = new Stopwatch();
        sw.Start();
        for (int i = 0; i < loops; i++)
            ASPXParse(i);
        sw.Stop();
        WriteTime("Total Elapsed Time: ", sw.ElapsedMilliseconds);
        return sw.ElapsedMilliseconds;
    }
    public static void ASPXParse(int loop)
    {
        StringWriter writer = new StringWriter();
        HttpContext.Current.Server.Execute("ASPNET.aspx", writer);
        writer.Close();
    }
}